"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var chai_1 = require("chai");
var marble_testing_1 = require("../helpers/marble-testing");
var operators_1 = require("rxjs/operators");
var rxjs_1 = require("rxjs");
describe('publishLast operator', function () {
    it('should emit last notification of a simple source Observable', function () {
        var source = (0, marble_testing_1.cold)('--1-2---3-4--5-|');
        var sourceSubs = '^              !';
        var published = source.pipe((0, operators_1.publishLast)());
        var expected = '---------------(5|)';
        (0, marble_testing_1.expectObservable)(published).toBe(expected);
        (0, marble_testing_1.expectSubscriptions)(source.subscriptions).toBe(sourceSubs);
        published.connect();
    });
    it('should return a ConnectableObservable-ish', function () {
        var source = (0, rxjs_1.of)(1).pipe((0, operators_1.publishLast)());
        (0, chai_1.expect)(typeof source._subscribe === 'function').to.be.true;
        (0, chai_1.expect)(typeof source.getSubject === 'function').to.be.true;
        (0, chai_1.expect)(typeof source.connect === 'function').to.be.true;
        (0, chai_1.expect)(typeof source.refCount === 'function').to.be.true;
    });
    it('should do nothing if connect is not called, despite subscriptions', function () {
        var source = (0, marble_testing_1.cold)('--1-2---3-4--5-|');
        var sourceSubs = [];
        var published = source.pipe((0, operators_1.publishLast)());
        var expected = '-';
        (0, marble_testing_1.expectObservable)(published).toBe(expected);
        (0, marble_testing_1.expectSubscriptions)(source.subscriptions).toBe(sourceSubs);
    });
    it('should multicast the same values to multiple observers', function () {
        var source = (0, marble_testing_1.cold)('-1-2-3----4-|');
        var sourceSubs = '^           !';
        var published = source.pipe((0, operators_1.publishLast)());
        var subscriber1 = (0, marble_testing_1.hot)('a|           ').pipe((0, operators_1.mergeMapTo)(published));
        var expected1 = '------------(4|)';
        var subscriber2 = (0, marble_testing_1.hot)('    b|       ').pipe((0, operators_1.mergeMapTo)(published));
        var expected2 = '    --------(4|)';
        var subscriber3 = (0, marble_testing_1.hot)('        c|   ').pipe((0, operators_1.mergeMapTo)(published));
        var expected3 = '        ----(4|)';
        (0, marble_testing_1.expectObservable)(subscriber1).toBe(expected1);
        (0, marble_testing_1.expectObservable)(subscriber2).toBe(expected2);
        (0, marble_testing_1.expectObservable)(subscriber3).toBe(expected3);
        (0, marble_testing_1.expectSubscriptions)(source.subscriptions).toBe(sourceSubs);
        published.connect();
    });
    it('should multicast an error from the source to multiple observers', function () {
        var source = (0, marble_testing_1.cold)('-1-2-3----4-#');
        var sourceSubs = '^           !';
        var published = source.pipe((0, operators_1.publishLast)());
        var subscriber1 = (0, marble_testing_1.hot)('a|           ').pipe((0, operators_1.mergeMapTo)(published));
        var expected1 = '------------#';
        var subscriber2 = (0, marble_testing_1.hot)('    b|       ').pipe((0, operators_1.mergeMapTo)(published));
        var expected2 = '    --------#';
        var subscriber3 = (0, marble_testing_1.hot)('        c|   ').pipe((0, operators_1.mergeMapTo)(published));
        var expected3 = '        ----#';
        (0, marble_testing_1.expectObservable)(subscriber1).toBe(expected1);
        (0, marble_testing_1.expectObservable)(subscriber2).toBe(expected2);
        (0, marble_testing_1.expectObservable)(subscriber3).toBe(expected3);
        (0, marble_testing_1.expectSubscriptions)(source.subscriptions).toBe(sourceSubs);
        published.connect();
    });
    it('should not cast any values to multiple observers, ' +
        'when source is unsubscribed explicitly and early', function () {
        var source = (0, marble_testing_1.cold)('-1-2-3----4-|');
        var sourceSubs = '^        !   ';
        var published = source.pipe((0, operators_1.publishLast)());
        var unsub = '         u   ';
        var subscriber1 = (0, marble_testing_1.hot)('a|           ').pipe((0, operators_1.mergeMapTo)(published));
        var expected1 = '----------   ';
        var subscriber2 = (0, marble_testing_1.hot)('    b|       ').pipe((0, operators_1.mergeMapTo)(published));
        var expected2 = '    ------   ';
        var subscriber3 = (0, marble_testing_1.hot)('        c|   ').pipe((0, operators_1.mergeMapTo)(published));
        var expected3 = '        --   ';
        (0, marble_testing_1.expectObservable)(subscriber1).toBe(expected1);
        (0, marble_testing_1.expectObservable)(subscriber2).toBe(expected2);
        (0, marble_testing_1.expectObservable)(subscriber3).toBe(expected3);
        (0, marble_testing_1.expectSubscriptions)(source.subscriptions).toBe(sourceSubs);
        var connection;
        (0, marble_testing_1.expectObservable)((0, marble_testing_1.hot)(unsub).pipe((0, operators_1.tap)(function () {
            connection.unsubscribe();
        }))).toBe(unsub);
        connection = published.connect();
    });
    it('should not break unsubscription chains when result is unsubscribed explicitly', function () {
        var source = (0, marble_testing_1.cold)('-1-2-3----4-|');
        var sourceSubs = '^        !   ';
        var published = source.pipe((0, operators_1.mergeMap)(function (x) { return (0, rxjs_1.of)(x); }), (0, operators_1.publishLast)());
        var subscriber1 = (0, marble_testing_1.hot)('a|           ').pipe((0, operators_1.mergeMapTo)(published));
        var expected1 = '----------   ';
        var subscriber2 = (0, marble_testing_1.hot)('    b|       ').pipe((0, operators_1.mergeMapTo)(published));
        var expected2 = '    ------   ';
        var subscriber3 = (0, marble_testing_1.hot)('        c|   ').pipe((0, operators_1.mergeMapTo)(published));
        var expected3 = '        --   ';
        var unsub = '         u   ';
        (0, marble_testing_1.expectObservable)(subscriber1).toBe(expected1);
        (0, marble_testing_1.expectObservable)(subscriber2).toBe(expected2);
        (0, marble_testing_1.expectObservable)(subscriber3).toBe(expected3);
        (0, marble_testing_1.expectSubscriptions)(source.subscriptions).toBe(sourceSubs);
        var connection;
        (0, marble_testing_1.expectObservable)((0, marble_testing_1.hot)(unsub).pipe((0, operators_1.tap)(function () {
            connection.unsubscribe();
        }))).toBe(unsub);
        connection = published.connect();
    });
    describe('with refCount()', function () {
        it('should connect when first subscriber subscribes', function () {
            var source = (0, marble_testing_1.cold)('-1-2-3----4-|');
            var sourceSubs = '   ^           !';
            var replayed = source.pipe((0, operators_1.publishLast)(), (0, operators_1.refCount)());
            var subscriber1 = (0, marble_testing_1.hot)('   a|           ').pipe((0, operators_1.mergeMapTo)(replayed));
            var expected1 = '   ------------(4|)';
            var subscriber2 = (0, marble_testing_1.hot)('       b|       ').pipe((0, operators_1.mergeMapTo)(replayed));
            var expected2 = '       --------(4|)';
            var subscriber3 = (0, marble_testing_1.hot)('           c|   ').pipe((0, operators_1.mergeMapTo)(replayed));
            var expected3 = '           ----(4|)';
            (0, marble_testing_1.expectObservable)(subscriber1).toBe(expected1);
            (0, marble_testing_1.expectObservable)(subscriber2).toBe(expected2);
            (0, marble_testing_1.expectObservable)(subscriber3).toBe(expected3);
            (0, marble_testing_1.expectSubscriptions)(source.subscriptions).toBe(sourceSubs);
        });
        it('should disconnect when last subscriber unsubscribes', function () {
            var source = (0, marble_testing_1.cold)('-1-2-3----4-|');
            var sourceSubs = '   ^        !   ';
            var replayed = source.pipe((0, operators_1.publishLast)(), (0, operators_1.refCount)());
            var subscriber1 = (0, marble_testing_1.hot)('   a|           ').pipe((0, operators_1.mergeMapTo)(replayed));
            var unsub1 = '          !     ';
            var expected1 = '   --------     ';
            var subscriber2 = (0, marble_testing_1.hot)('       b|       ').pipe((0, operators_1.mergeMapTo)(replayed));
            var unsub2 = '            !   ';
            var expected2 = '       ------   ';
            (0, marble_testing_1.expectObservable)(subscriber1, unsub1).toBe(expected1);
            (0, marble_testing_1.expectObservable)(subscriber2, unsub2).toBe(expected2);
            (0, marble_testing_1.expectSubscriptions)(source.subscriptions).toBe(sourceSubs);
        });
        it('should NOT be retryable', function () {
            var source = (0, marble_testing_1.cold)('-1-2-3----4-#');
            var sourceSubs = '^           !';
            var published = source.pipe((0, operators_1.publishLast)(), (0, operators_1.refCount)(), (0, operators_1.retry)(3));
            var subscriber1 = (0, marble_testing_1.hot)('a|           ').pipe((0, operators_1.mergeMapTo)(published));
            var expected1 = '------------#';
            var subscriber2 = (0, marble_testing_1.hot)('    b|       ').pipe((0, operators_1.mergeMapTo)(published));
            var expected2 = '    --------#';
            var subscriber3 = (0, marble_testing_1.hot)('        c|   ').pipe((0, operators_1.mergeMapTo)(published));
            var expected3 = '        ----#';
            (0, marble_testing_1.expectObservable)(subscriber1).toBe(expected1);
            (0, marble_testing_1.expectObservable)(subscriber2).toBe(expected2);
            (0, marble_testing_1.expectObservable)(subscriber3).toBe(expected3);
            (0, marble_testing_1.expectSubscriptions)(source.subscriptions).toBe(sourceSubs);
        });
    });
    it('should multicast an empty source', function () {
        var source = (0, marble_testing_1.cold)('|');
        var sourceSubs = '(^!)';
        var published = source.pipe((0, operators_1.publishLast)());
        var expected = '|';
        (0, marble_testing_1.expectObservable)(published).toBe(expected);
        (0, marble_testing_1.expectSubscriptions)(source.subscriptions).toBe(sourceSubs);
        published.connect();
    });
    it('should multicast a never source', function () {
        var source = (0, marble_testing_1.cold)('-');
        var sourceSubs = '^';
        var published = source.pipe((0, operators_1.publishLast)());
        var expected = '-';
        (0, marble_testing_1.expectObservable)(published).toBe(expected);
        (0, marble_testing_1.expectSubscriptions)(source.subscriptions).toBe(sourceSubs);
        published.connect();
    });
    it('should multicast a throw source', function () {
        var source = (0, marble_testing_1.cold)('#');
        var sourceSubs = '(^!)';
        var published = source.pipe((0, operators_1.publishLast)());
        var expected = '#';
        (0, marble_testing_1.expectObservable)(published).toBe(expected);
        (0, marble_testing_1.expectSubscriptions)(source.subscriptions).toBe(sourceSubs);
        published.connect();
    });
    it('should multicast one observable to multiple observers', function (done) {
        var results1 = [];
        var results2 = [];
        var subscriptions = 0;
        var source = new rxjs_1.Observable(function (observer) {
            subscriptions++;
            observer.next(1);
            observer.next(2);
            observer.next(3);
            observer.next(4);
            observer.complete();
        });
        var connectable = source.pipe((0, operators_1.publishLast)());
        connectable.subscribe(function (x) {
            results1.push(x);
        });
        connectable.subscribe(function (x) {
            results2.push(x);
        });
        (0, chai_1.expect)(results1).to.deep.equal([]);
        (0, chai_1.expect)(results2).to.deep.equal([]);
        connectable.connect();
        (0, chai_1.expect)(results1).to.deep.equal([4]);
        (0, chai_1.expect)(results2).to.deep.equal([4]);
        (0, chai_1.expect)(subscriptions).to.equal(1);
        done();
    });
    it('should be referentially-transparent', function () {
        var source1 = (0, marble_testing_1.cold)('-1-2-3-4-5-|');
        var source1Subs = '^          !';
        var expected1 = '-----------(5|)';
        var source2 = (0, marble_testing_1.cold)('-6-7-8-9-0-|');
        var source2Subs = '^          !';
        var expected2 = '-----------(0|)';
        var partialPipeLine = (0, rxjs_1.pipe)((0, operators_1.publishLast)());
        var published1 = source1.pipe(partialPipeLine);
        var published2 = source2.pipe(partialPipeLine);
        (0, marble_testing_1.expectObservable)(published1).toBe(expected1);
        (0, marble_testing_1.expectSubscriptions)(source1.subscriptions).toBe(source1Subs);
        (0, marble_testing_1.expectObservable)(published2).toBe(expected2);
        (0, marble_testing_1.expectSubscriptions)(source2.subscriptions).toBe(source2Subs);
        published1.connect();
        published2.connect();
    });
});
//# sourceMappingURL=publishLast-spec.js.map